@@ -1,20 +1,17 @@
 """Server for mypy daemon mode.
 
-Only supports UNIX-like systems.
-
 This implements a daemon process which keeps useful state in memory
 to enable fine-grained incremental reprocessing of changes.
 """
 
 import argparse
 import base64
+import io
 import json
 import os
 import pickle
-import random
 import subprocess
 import sys
-import tempfile
 import time
 import traceback
 
@@ -25,13 +22,14 @@ import mypy.errors
 import mypy.main
 from mypy.find_sources import create_source_list, InvalidSourceList
 from mypy.server.update import FineGrainedBuildManager
-from mypy.dmypy_util import STATUS_FILE, receive
-from mypy.ipc import IPCServer, IPCException
+from mypy.dmypy_util import receive
+from mypy.ipc import IPCServer
 from mypy.fscache import FileSystemCache
 from mypy.fswatcher import FileSystemWatcher, FileData
 from mypy.modulefinder import BuildSource, compute_search_paths
 from mypy.options import Options
 from mypy.typestate import reset_global_state
+from mypy.util import redirect_stderr, redirect_stdout
 from mypy.version import __version__
 
 
@@ -45,6 +43,7 @@ if sys.platform == 'win32':
     from subprocess import STARTUPINFO
 
     def daemonize(options: Options,
+                  status_file: str,
                   timeout: Optional[int] = None,
                   log_file: Optional[str] = None) -> int:
         """Create the daemon process via "dmypy daemon" and pass options via command line
@@ -57,12 +56,12 @@ if sys.platform == 'win32':
 
         It also pickles the options to be unpickled by mypy.
         """
-        command = [sys.executable, '-m', 'mypy.dmypy', 'daemon']
+        command = [sys.executable, '-m', 'mypy.dmypy', '--status-file', status_file, 'daemon']
         pickeled_options = pickle.dumps((options.snapshot(), timeout, log_file))
         command.append('--options-data="{}"'.format(base64.b64encode(pickeled_options).decode()))
-        info = STARTUPINFO(dwFlags=0x1,  # STARTF_USESHOWWINDOW aka use wShowWindow's value
-                           wShowWindow=0,  # SW_HIDE aka make the window invisible
-                           )
+        info = STARTUPINFO()
+        info.dwFlags = 0x1  # STARTF_USESHOWWINDOW aka use wShowWindow's value
+        info.wShowWindow = 0  # SW_HIDE aka make the window invisible
         try:
             subprocess.Popen(command,
                              creationflags=0x10,  # CREATE_NEW_CONSOLE
@@ -118,6 +117,7 @@ else:
             os._exit(1)
 
     def daemonize(options: Options,
+                  status_file: str,
                   timeout: Optional[int] = None,
                   log_file: Optional[str] = None) -> int:
         """Run the mypy daemon in a grandchild of the current process
@@ -125,11 +125,11 @@ else:
         Return 0 for success, exit status for failure, negative if
         subprocess killed by signal.
         """
-        return _daemonize_cb(Server(options, timeout).serve, log_file)
+        return _daemonize_cb(Server(options, status_file, timeout).serve, log_file)
 
 # Server code.
 
-CONNECTION_NAME = 'dmypy.sock'  # type: Final
+CONNECTION_NAME = 'dmypy'  # type: Final
 
 
 def process_start_options(flags: List[str], allow_sources: bool) -> Options:
@@ -145,8 +145,6 @@ def process_start_options(flags: List[str], allow_sources: bool) -> Options:
                  "pass it to check/recheck instead")
     if not options.incremental:
         sys.exit("dmypy: start/restart should not disable incremental mode")
-    if options.quick_and_dirty:
-        sys.exit("dmypy: start/restart should not specify quick_and_dirty mode")
     # Our file change tracking can't yet handle changes to files that aren't
     # specified in the sources list.
     if options.follow_imports not in ('skip', 'error'):
@@ -165,6 +163,7 @@ class Server:
     # serve() is called in the grandchild (by daemonize()).
 
     def __init__(self, options: Options,
+                 status_file: str,
                  timeout: Optional[int] = None) -> None:
         """Initialize the server with the desired mypy flags."""
         self.options = options
@@ -173,8 +172,8 @@ class Server:
         self.timeout = timeout
         self.fine_grained_manager = None  # type: Optional[FineGrainedBuildManager]
 
-        if os.path.isfile(STATUS_FILE):
-            os.unlink(STATUS_FILE)
+        if os.path.isfile(status_file):
+            os.unlink(status_file)
 
         self.fscache = FileSystemCache()
 
@@ -190,13 +189,21 @@ class Server:
         # Fine-grained incremental doesn't support general partial types
         # (details in https://github.com/python/mypy/issues/4492)
         options.local_partial_types = True
+        self.status_file = status_file
+
+    def _response_metadata(self) -> Dict[str, str]:
+        py_version = '{}.{}'.format(self.options.python_version[0], self.options.python_version[1])
+        return {
+            'platform': self.options.platform,
+            'python_version': py_version,
+        }
 
     def serve(self) -> None:
         """Serve requests, synchronously (no thread or fork)."""
         command = None
         try:
             server = IPCServer(CONNECTION_NAME, self.timeout)
-            with open(STATUS_FILE, 'w') as f:
+            with open(self.status_file, 'w') as f:
                 json.dump({'pid': os.getpid(), 'connection_name': server.connection_name}, f)
                 f.write('\n')  # I like my JSON with a trailing newline
             while True:
@@ -217,9 +224,11 @@ class Server:
                                 # If we are crashing, report the crash to the client
                                 tb = traceback.format_exception(*sys.exc_info())
                                 resp = {'error': "Daemon crashed!\n" + "".join(tb)}
+                                resp.update(self._response_metadata())
                                 server.write(json.dumps(resp).encode('utf8'))
                                 raise
                     try:
+                        resp.update(self._response_metadata())
                         server.write(json.dumps(resp).encode('utf8'))
                     except OSError:
                         pass  # Maybe the client hung up
@@ -233,7 +242,7 @@ class Server:
             # that could cause us to remove a future server's
             # status file.)
             if command != 'stop':
-                os.unlink(STATUS_FILE)
+                os.unlink(self.status_file)
             try:
                 server.cleanup()  # try to remove the socket dir on Linux
             except OSError:
@@ -265,17 +274,25 @@ class Server:
         # RPC. Otherwise a race condition exists where a subsequent
         # command can see a status file from a dying server and think
         # it is a live one.
-        os.unlink(STATUS_FILE)
+        os.unlink(self.status_file)
         return {}
 
     def cmd_run(self, version: str, args: Sequence[str]) -> Dict[str, object]:
         """Check a list of files, triggering a restart if needed."""
         try:
-            sources, options = mypy.main.process_options(
-                ['-i'] + list(args),
-                require_targets=True,
-                server_options=True,
-                fscache=self.fscache)
+            # Process options can exit on improper arguments, so we need to catch that and
+            # capture stderr so the client can report it
+            stderr = io.StringIO()
+            stdout = io.StringIO()
+            with redirect_stderr(stderr):
+                with redirect_stdout(stdout):
+                    sources, options = mypy.main.process_options(
+                        ['-i'] + list(args),
+                        require_targets=True,
+                        server_options=True,
+                        fscache=self.fscache,
+                        program='mypy-daemon',
+                        header=argparse.SUPPRESS)
             # Signal that we need to restart if the options have changed
             if self.options_snapshot != options.snapshot():
                 return {'restart': 'configuration changed'}
@@ -289,6 +306,8 @@ class Server:
                     return {'restart': 'plugins changed'}
         except InvalidSourceList as err:
             return {'out': '', 'err': str(err), 'status': 2}
+        except SystemExit as e:
+            return {'out': stdout.getvalue(), 'err': stderr.getvalue(), 'status': e.code}
         return self.check(sources)
 
     def cmd_check(self, files: Sequence[str]) -> Dict[str, object]:
@@ -327,6 +346,7 @@ class Server:
         manager.log("fine-grained increment: cmd_recheck: {:.3f}s".format(t1 - t0))
         res = self.fine_grained_increment(sources, remove, update)
         self.fscache.flush()
+        self.update_stats(res)
         return res
 
     def check(self, sources: List[BuildSource]) -> Dict[str, Any]:
@@ -336,11 +356,21 @@ class Server:
         else:
             res = self.fine_grained_increment(sources)
         self.fscache.flush()
+        self.update_stats(res)
         return res
 
+    def update_stats(self, res: Dict[str, Any]) -> None:
+        if self.fine_grained_manager:
+            manager = self.fine_grained_manager.manager
+            manager.dump_stats()
+            res['stats'] = manager.stats
+            manager.stats = {}
+
     def initialize_fine_grained(self, sources: List[BuildSource]) -> Dict[str, Any]:
         self.fswatcher = FileSystemWatcher(self.fscache)
+        t0 = time.time()
         self.update_sources(sources)
+        t1 = time.time()
         try:
             result = mypy.build.build(sources=sources,
                                       options=self.options,
@@ -360,6 +390,7 @@ class Server:
         # the typechecking on the updated files yet.
         # Run a fine-grained update starting from the cached data
         if result.used_cache:
+            t2 = time.time()
             # Pull times and hashes out of the saved_cache and stick them into
             # the fswatcher, so we pick up the changes.
             for state in self.fine_grained_manager.graph.values():
@@ -378,8 +409,16 @@ class Server:
                     assert state.path is not None
                     changed.append((state.id, state.path))
 
+            t3 = time.time()
             # Run an update
             messages = self.fine_grained_manager.update(changed, removed)
+            t4 = time.time()
+            self.fine_grained_manager.manager.add_stats(
+                update_sources_time=t1 - t0,
+                build_time=t2 - t1,
+                find_changes_time=t3 - t2,
+                fg_update_time=t4 - t3,
+                files_changed=len(removed) + len(changed))
         else:
             # Stores the initial state of sources as a side effect.
             self.fswatcher.find_changed()
@@ -415,6 +454,11 @@ class Server:
         messages = self.fine_grained_manager.update(changed, removed)
         t2 = time.time()
         manager.log("fine-grained increment: update: {:.3f}s".format(t2 - t1))
+        manager.add_stats(
+            find_changes_time=t1 - t0,
+            fg_update_time=t2 - t1,
+            files_changed=len(removed) + len(changed))
+
         status = 1 if messages else 0
         self.previous_sources = sources
         return {'out': ''.join(s + '\n' for s in messages), 'err': '', 'status': status}
