@@ -1,9 +1,10 @@
 """Hack for handling non-mypyc compiled plugins with a mypyc-compiled mypy"""
 
-from typing import Optional, Callable, Any
+from typing import Optional, Callable, Any, Dict
 from mypy.options import Options
 from mypy.types import Type, CallableType
-
+from mypy.nodes import SymbolTableNode, MypyFile
+from mypy.lookup import lookup_fully_qualified
 
 MYPY = False
 if MYPY:
@@ -29,6 +30,14 @@ class InterpretedPlugin:
     def __init__(self, options: Options) -> None:
         self.options = options
         self.python_version = options.python_version
+        self._modules = None  # type: Optional[Dict[str, MypyFile]]
+
+    def set_modules(self, modules: Dict[str, MypyFile]) -> None:
+        self._modules = modules
+
+    def lookup_fully_qualified(self, fullname: str) -> Optional[SymbolTableNode]:
+        assert self._modules is not None
+        return lookup_fully_qualified(fullname, self._modules)
 
     def get_type_analyze_hook(self, fullname: str
                               ) -> Optional[Callable[['mypy.plugin.AnalyzeTypeContext'], Type]]:
