@@ -13,14 +13,13 @@ from typing import Any, Dict, List, Mapping, Optional, Tuple
 
 from mypy import build
 from mypy import defaults
-from mypy import experiments
+from mypy import state
 from mypy import util
 from mypy.modulefinder import BuildSource, FindModuleCache, mypy_path, SearchPaths
 from mypy.find_sources import create_source_list, InvalidSourceList
 from mypy.fscache import FileSystemCache
 from mypy.errors import CompileError
 from mypy.options import Options, BuildType, PER_MODULE_OPTIONS
-from mypy.report import reporter_classes
 
 from mypy.version import __version__
 
@@ -102,7 +101,9 @@ def main(script_path: Optional[str], args: Optional[List[str]] = None) -> None:
               file=sys.stderr)
     if options.junit_xml:
         t1 = time.time()
-        util.write_junit_xml(t1 - t0, serious, messages, options.junit_xml)
+        py_version = '{}.{}'.format(options.python_version[0], options.python_version[1])
+        util.write_junit_xml(t1 - t0, serious, messages, options.junit_xml,
+                             py_version, options.platform)
 
     if MEM_PROFILE:
         from mypy.memprofile import print_memory_profile
@@ -121,16 +122,6 @@ def main(script_path: Optional[str], args: Optional[List[str]] = None) -> None:
         sys.exit(code)
 
 
-def readlinkabs(link: str) -> str:
-    """Return an absolute path to symbolic link destination."""
-    # Adapted from code by Greg Smith.
-    assert os.path.islink(link)
-    path = os.readlink(link)
-    if os.path.isabs(path):
-        return path
-    return os.path.join(os.path.dirname(link), path)
-
-
 class SplitNamespace(argparse.Namespace):
     def __init__(self, standard_namespace: object, alt_namespace: object, alt_prefix: str) -> None:
         self.__dict__['_standard_namespace'] = standard_namespace
@@ -227,17 +218,6 @@ def python_executable_prefix(v: str) -> List[str]:
         return ['python{}'.format(v)]
 
 
-def _python_version_from_executable(python_executable: str) -> Tuple[int, int]:
-    try:
-        check = subprocess.check_output([python_executable, '-c',
-                                         'import sys; print(repr(sys.version_info[:2]))'],
-                                        stderr=subprocess.STDOUT).decode()
-        return ast.literal_eval(check)
-    except (subprocess.CalledProcessError, FileNotFoundError):
-        raise PythonExecutableInferenceError(
-            'invalid Python executable {}'.format(python_executable))
-
-
 def _python_executable_from_version(python_version: Tuple[int, int]) -> str:
     if sys.version_info[:2] == python_version:
         return sys.executable
@@ -253,37 +233,25 @@ def _python_executable_from_version(python_version: Tuple[int, int]) -> str:
             ' perhaps try --python-executable, or --no-site-packages?'.format(python_version))
 
 
-def infer_python_version_and_executable(options: Options,
-                                        special_opts: argparse.Namespace) -> None:
-    """Infer the Python version or executable from each other. Check they are consistent.
+def infer_python_executable(options: Options,
+                            special_opts: argparse.Namespace) -> None:
+    """Infer the Python executable from the given version.
 
-    This function mutates options based on special_opts to infer the correct Python version and
-    executable to use.
+    This function mutates options based on special_opts to infer the correct Python executable
+    to use.
     """
-    # Infer Python version and/or executable if one is not given
-
     # TODO: (ethanhs) Look at folding these checks and the site packages subprocess calls into
     # one subprocess call for speed.
-    if special_opts.python_executable is not None and special_opts.python_version is not None:
-        py_exe_ver = _python_version_from_executable(special_opts.python_executable)
-        if py_exe_ver != special_opts.python_version:
-            raise PythonExecutableInferenceError(
-                'Python version {} did not match executable {}, got version {}.'.format(
-                    special_opts.python_version, special_opts.python_executable, py_exe_ver
-                ))
-        else:
-            options.python_version = special_opts.python_version
-            options.python_executable = special_opts.python_executable
-    elif special_opts.python_executable is None and special_opts.python_version is not None:
-        options.python_version = special_opts.python_version
-        py_exe = None
+
+    # Use the command line specified executable, or fall back to one set in the
+    # config file. If an executable is not specified, infer it from the version
+    # (unless no_executable is set)
+    python_executable = special_opts.python_executable or options.python_executable
+
+    if python_executable is None:
         if not special_opts.no_executable:
-            py_exe = _python_executable_from_version(special_opts.python_version)
-        options.python_executable = py_exe
-    elif special_opts.python_version is None and special_opts.python_executable is not None:
-        options.python_version = _python_version_from_executable(
-            special_opts.python_executable)
-        options.python_executable = special_opts.python_executable
+            python_executable = _python_executable_from_version(options.python_version)
+    options.python_executable = python_executable
 
 
 HEADER = """%(prog)s [-h] [-v] [-V] [more options; see below]
@@ -321,6 +289,8 @@ def process_options(args: List[str],
                     require_targets: bool = True,
                     server_options: bool = False,
                     fscache: Optional[FileSystemCache] = None,
+                    program: str = 'mypy',
+                    header: str = HEADER,
                     ) -> Tuple[List[BuildSource], Options]:
     """Parse command line arguments.
 
@@ -328,8 +298,8 @@ def process_options(args: List[str],
     call fscache.set_package_root() to set the cache's package root.
     """
 
-    parser = argparse.ArgumentParser(prog='mypy',
-                                     usage=HEADER,
+    parser = argparse.ArgumentParser(prog=program,
+                                     usage=header,
                                      description=DESCRIPTION,
                                      epilog=FOOTER,
                                      fromfile_prefix_chars='@',
@@ -570,6 +540,9 @@ def process_options(args: List[str],
         '--cache-dir', action='store', metavar='DIR',
         help="Store module cache info in the given folder in incremental mode "
              "(defaults to '{}')".format(defaults.CACHE_DIR))
+    add_invertible_flag('--sqlite-cache', default=False,
+                        help="Use a sqlite database to store the cache",
+                        group=incremental_group)
     incremental_group.add_argument(
         '--cache-fine-grained', action='store_true',
         help="Include fine-grained dependency information in the cache for the mypy daemon")
@@ -626,7 +599,7 @@ def process_options(args: List[str],
     report_group = parser.add_argument_group(
         title='Report generation',
         description='Generate a report in the specified format.')
-    for report_type in sorted(reporter_classes):
+    for report_type in sorted(defaults.REPORTER_NAMES):
         report_group.add_argument('--%s-report' % report_type.replace('_', '-'),
                                   metavar='DIR',
                                   dest='special-opts:%s_report' % report_type)
@@ -690,10 +663,6 @@ def process_options(args: List[str],
     parser.add_argument('--cache-map', nargs='+', dest='special-opts:cache_map',
                         help=argparse.SUPPRESS)
 
-    # deprecated options
-    parser.add_argument('--quick-and-dirty', action='store_true',
-                        help=argparse.SUPPRESS)
-
     # options specifying code to check
     code_group = parser.add_argument_group(
         title="Running code",
@@ -739,13 +708,11 @@ def process_options(args: List[str],
     special_opts = argparse.Namespace()
     parser.parse_args(args, SplitNamespace(options, special_opts, 'special-opts:'))
 
-    # Process deprecated options
-    if options.quick_and_dirty:
-        print("Warning: --quick-and-dirty is deprecated.  It will disappear in the next release.",
-              file=sys.stderr)
-
+    # The python_version is either the default, which can be overridden via a config file,
+    # or stored in special_opts and is passed via the command line.
+    options.python_version = special_opts.python_version or options.python_version
     try:
-        infer_python_version_and_executable(options, special_opts)
+        infer_python_executable(options, special_opts)
     except PythonExecutableInferenceError as e:
         parser.error(str(e))
 
@@ -773,11 +740,11 @@ def process_options(args: List[str],
         # TODO: Deprecate, then kill this flag
         options.strict_optional = True
     if special_opts.find_occurrences:
-        experiments.find_occurrences = special_opts.find_occurrences.split('.')
-        assert experiments.find_occurrences is not None
-        if len(experiments.find_occurrences) < 2:
+        state.find_occurrences = special_opts.find_occurrences.split('.')
+        assert state.find_occurrences is not None
+        if len(state.find_occurrences) < 2:
             parser.error("Can only find occurrences of class members.")
-        if len(experiments.find_occurrences) != 2:
+        if len(state.find_occurrences) != 2:
             parser.error("Can only find occurrences of non-nested class members.")
 
     # Set reports.
@@ -793,11 +760,10 @@ def process_options(args: List[str],
 
     # Process --cache-map.
     if special_opts.cache_map:
-        process_cache_map(parser, special_opts, options)
+        if options.sqlite_cache:
+            parser.error("--cache-map is incompatible with --sqlite-cache")
 
-    # Let quick_and_dirty imply incremental.
-    if options.quick_and_dirty:
-        options.incremental = True
+        process_cache_map(parser, special_opts, options)
 
     # Let logical_deps imply cache_fine_grained (otherwise the former is useless).
     if options.logical_deps:
@@ -995,7 +961,7 @@ def parse_section(prefix: str, template: Options,
             if dv is None:
                 if key.endswith('_report'):
                     report_type = key[:-7].replace('_', '-')
-                    if report_type in reporter_classes:
+                    if report_type in defaults.REPORTER_NAMES:
                         report_dirs[report_type] = section[key]
                     else:
                         print("%s: Unrecognized report type: %s" % (prefix, key),
