@@ -19,7 +19,7 @@ those with platform checks.
 
 from typing import List, Tuple
 
-from mypy import experiments
+from mypy import state
 from mypy.nodes import (
     MypyFile, SymbolTable, SymbolTableNode, Var, Block, AssignmentStmt, FuncDef, Decorator,
     ClassDef, TypeInfo, ImportFrom, Import, ImportAll, IfStmt, WhileStmt, ForStmt, WithStmt,
@@ -27,9 +27,8 @@ from mypy.nodes import (
     MODULE_REF, implicit_module_attrs, AssertStmt,
 )
 from mypy.types import Type, UnboundType, UnionType, AnyType, TypeOfAny, NoneTyp, CallableType
-from mypy.semanal import (
-    SemanticAnalyzerPass2, infer_reachability_of_if_statement, assert_will_always_fail,
-)
+from mypy.semanal import SemanticAnalyzerPass2
+from mypy.reachability import infer_reachability_of_if_statement, assert_will_always_fail
 from mypy.semanal_shared import create_indirect_imported_name
 from mypy.options import Options
 from mypy.sametypes import is_same_type
@@ -77,7 +76,7 @@ class SemanticAnalyzerPass1(NodeVisitor[None]):
 
         defs = file.defs
 
-        with experiments.strict_optional_set(options.strict_optional):
+        with state.strict_optional_set(options.strict_optional):
             # Add implicit definitions of module '__name__' etc.
             for name, t in implicit_module_attrs.items():
                 # unicode docstrings should be accepted in Python 2
